<?php
// $Id: subscriptions.admin.inc,v 1.1.2.9 2007/12/06 21:16:14 chx Exp $

/**
 * Admin settings
 */
function subscriptions_settings_form() {
  $form['sub_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('General settings'),
      '#weight' => -10,
  );
  $form['sub_settings']['subscriptions_sendself'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Notify poster of own posts'),
    '#default_value' => variable_get('subscriptions_sendself', 0),
    '#description'   => t("Notifies a node poster about their own posts.  Useful principally during testing.  Default is OFF."),
  );
  $form['sub_settings']['subscriptions_watchgood'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display watchdog entries for successful mailings'),
    '#default_value' => variable_get('subscriptions_watchgood', 1),
    '#description'   => t('Inserts notification of successful mailings in the watchdog log.  Default is ON.'),
  );
  $form['sub_settings']['subscriptions_testpost'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Test held posts prior to sending'),
    '#default_value' => variable_get('subscriptions_testpost', 0),
    '#description'   => t('Tests to see if a post about to be sent by cron is still active.  Adds a small amount of overhead.  Default is OFF.'),
  );
  $form['sub_settings']['subscriptions_autoset'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Set all users to "autosubscribe" by default'),
    '#default_value' => variable_get('subscriptions_autoset', 0),
    '#description'   => t('Sets each users "autosubscribe" profile option. Default is OFF.'),
  );
  $form['sub_settings']['subscriptions_link_teaser'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Show subscribe link with teaser'),
    '#default_value' => variable_get('subscriptions_link_teaser', 1),
    '#description'   => t('Uncheck to show link only in node view.'),
  );

  return system_settings_form($form);
}

/* ******************************************************* */
/*  user screens: display, edit functions */
/* ******************************************************* */

function theme_subscriptions_form_table($element) {
  $output = '';
  $rows = array();
  $header = array('&nbsp;', t('Send Interval'), t('On updates'), t('On comments'), t('Author'));
  foreach (element_children($element['checkboxes']) as $key) {
    foreach (element_children($element['checkboxes'][$key]) as $key1) {
      $rows[] = array(
        array('data' => drupal_render($element['checkboxes'][$key][$key1]), 'width' => '20%'),
        drupal_render($element['selects'][$key][$key1]),
        drupal_render($element['send_updates'][$key][$key1]),
        drupal_render($element['send_comments'][$key][$key1]),
        drupal_render($element['author'][$key][$key1]),
      );
    }
  }
  if ($rows) {
    $output .= theme('table', $header, $rows);
    $output .= drupal_render($element);
  }
  return $output;
}


function subscriptions_page_form_submit($form_id, $form_values) {
  global $user;
  $uid = $form_values['uid'];
  $module = $form_values['module'];
  $field = $form_values['field'];
  foreach ($form_values as $element_values) {
    if (isset($element_values['defaults']['selects'])) {
      $defaults = $element_values['defaults'];
      foreach ($element_values['checkboxes'] as $value => $bundle) {
        foreach ($bundle as $author_uid => $set_to_enabled) {
          if ($set_to_enabled && (
              !isset($defaults['checkboxes'][$value][$author_uid]) ||
              $element_values['selects'][$value][$author_uid] != $defaults['selects'][$value][$author_uid] ||
              $element_values['send_updates'][$value][$author_uid] != $defaults['send_updates'][$value][$author_uid] ||
              $element_values['send_comments'][$value][$author_uid] != $defaults['send_comments'][$value][$author_uid])) {
            $send_updates = ($element_values['send_updates'][$value][$author_uid] ? 1 : 0);
            $send_comments = ($element_values['send_comments'][$value][$author_uid] ? 1 : 0);
            subscriptions_write($module, $field, $value, $author_uid, $uid, $element_values['selects'][$value][$author_uid], $send_updates, $send_comments);
            drupal_set_message(t('Your subscription was activated.'));
          }
          elseif (!$set_to_enabled && isset($defaults['checkboxes'][$value][$author_uid])) {
            // It was enabled but now it is not. Delete.
            db_query("DELETE FROM {subscriptions} WHERE module = '%s' AND field = '%s' AND value = %d AND author_uid = %d AND recipient_uid = %d", $module, $field, $value, $author_uid, $uid);
            drupal_set_message(t('Your subscription was deactivated.'));
          }
        }
      }
    }
  }
}

/**
 * displays subscribed content data on user and subuscription pages
 * @ TODO clean up all of these parts
 */
function subscriptions_page($account, $stype) {
  $stypes = subscriptions_types();
  if (!isset($stypes[$stype])) {
    drupal_not_found();
    return;
  }
  $callback = $stypes[$stype]['page'];
  $output = function_exists($callback) ? $callback($account) : '';
  drupal_add_link(array(
    'rel' => 'alternate',
    'type' => 'application/rss+xml',
    'title' => t("!name Subscriptions", array('!name' => $user->name)),
    'href' => url('subscriptions/feed'),
  ));
  drupal_set_title(check_plain($account->name));
  return theme('box', '', $output) . theme('xml_icon', url("subscriptions/feed"));
}

function subscriptions_page_user_overview($account) {
  // Build summary
  $count = array();
  $result = db_query("SELECT module, field, count(1) as number FROM {subscriptions} WHERE recipient_uid = %d GROUP BY module, field", $account->uid);
  while ($subs = db_fetch_object($result)) {
    $count[$subs->module][$subs->field] =  $subs->number;
  }

  $header = array(t('Type'), t('Number'));
  foreach (subscriptions_types() as $stype => $type) {
    $has_access = user_access($type['access'], $account);
    if ($has_access || user_access('administer user subscriptions')) {
      $rows[] = array(
        ($has_access ? l($type['title'], "user/$account->uid/subscriptions/$stype") : $type['title']),
        isset($count[$type['fields'][0]][$type['fields'][1]]) ? $count[$type['fields'][0]][$type['fields'][1]] : 0,
      );
    }
  }
  $output .= theme('table', $header, $rows);
  drupal_set_title(check_plain($account->name));
  return $output;
}

function subscriptions_intervals() {
  $default = '';
  foreach (_subscriptions_send_intervals() as $interval => $text) {
    $default .= "$interval|$text\n";
  }
  $form['intervals'] = array(
    '#type' => 'textarea',
    '#cols' => 40,
    '#default_value' => $default,
    '#description' => t('Every line has the format of seconds|human readable text'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

function subscriptions_intervals_submit($form_id, $form_values) {
  $intervals = array();
  foreach (explode("\n", $form_values['intervals']) as $line) {
    if ($line) {
      $v = explode('|', $line);
      $intervals[$v[0]] = $v[1];
    }
  }
  variable_set('subscriptions_send_intervals', $intervals);
}

function subscriptions_add_form($stype, $sid, $uid = NULL) {
  global $user;
  $form['sid'] = array('#type' => 'value', '#value' => $sid);
  $form['uid'] = array('#type' => 'value', '#value' => isset($uid) ? $uid : $user->uid);
  $form['stype'] = array('#type' => 'value', '#value' => $stype);
  $form['send_interval'] = array(
    '#title' => t('Send interval'),
    '#type' => 'select',
    '#options' => _subscriptions_send_intervals(),
    '#default_value' => 1,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Subscribe'),
  );
  return $form;
}

function subscriptions_add_form_submit($form_id, $form_values) {
  if ($a = module_invoke_all('subscriptions', 'stype', $form_values['stype'])) {
    list($module, $field) = $a;
  }
  subscriptions_write_subscription($module, $field, $form_values['sid'], $form_values['uid'], $form_values['send_interval']);
  drupal_set_message(t('Your subscription was activated.'));
}

/**
 * List of send intervals
 */
function _subscriptions_send_intervals() {
  return variable_get('subscriptions_send_intervals',
    array(
      1 => t('As soon as possible'),
      900 => t('Every 15 minutes'),
      3600 => t('Hourly'),
      10800 => t('Every three hours'),
      86400 => t('Daily'),
    )
  );
}

function subscriptions_form_helper(&$form, &$defaults, $author_uid, $key, $title, $subscription) {
  static $intervals;
  if (!isset($intervals)) {
    $intervals = _subscriptions_send_intervals();
  }
  if (($author_uid == $subscription['author_uid'])) {
    $defaults['checkboxes'][$key][$author_uid] = 1;
  }
  $defaults['selects'][$key][$author_uid] = $subscription['send_interval'];
  if ($subscription['send_comments']) {
    $defaults['send_comments'][$key][$author_uid] = 1;
  }
  if ($subscription['send_updates']) {
    $defaults['send_updates'][$key][$author_uid] = 1;
  }
  if ($subscription['author_uid'] > 0) {
    $author_value = check_plain(db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $subscription['author_uid'])));
  }
  else {
    $author_value = '';
  }
  $form['author'][$key][$author_uid] = array(
    '#value' => $author_value,
  );
  $form['selects'][$key][$author_uid] = array(
    '#type' => 'select',
    '#options' => $intervals,
    '#default_value' => $defaults['selects'][$key][$author_uid],
  );
  foreach (array('checkboxes', 'send_comments', 'send_updates') as $index => $name) {
    $form[$name][$key][$author_uid] = array(
      '#type' => 'checkbox',
      '#title' => $index ? '' : $title,
      '#default_value' => $defaults[$name][$key][$author_uid],
    );
  }
}